[llvm-commits] [llvm] r63898 - in /llvm/trunk: lib/Transforms/Scalar/CondPropagate.cpp test/Transforms/CondProp/basictest-dbg.ll

Devang Patel dpatel at apple.com
Thu Feb 5 15:32:53 PST 2009


Author: dpatel
Date: Thu Feb  5 17:32:52 2009
New Revision: 63898

URL: http://llvm.org/viewvc/llvm-project?rev=63898&view=rev
Log:

Ignore dbg intrinsics while propagating conditional expression info. Take 2.

Added:
    llvm/trunk/test/Transforms/CondProp/basictest-dbg.ll
Modified:
    llvm/trunk/lib/Transforms/Scalar/CondPropagate.cpp

Modified: llvm/trunk/lib/Transforms/Scalar/CondPropagate.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Transforms/Scalar/CondPropagate.cpp?rev=63898&r1=63897&r2=63898&view=diff

==============================================================================
--- llvm/trunk/test/Transforms/CondProp/basictest-dbg.ll (added)
+++ llvm/trunk/test/Transforms/CondProp/basictest-dbg.ll Thu Feb  5 17:32:52 2009
@@ -0,0 +1,45 @@
+; RUN: llvm-as < %s | opt -condprop | llvm-dis | \
+; RUN:    not grep {br label}
+
+        %llvm.dbg.anchor.type = type { i32, i32 }
+        %llvm.dbg.compile_unit.type = type { i32, { }*, i32, i8*, i8*, i8*, i1, i1, i8* }
+
+ at llvm.dbg.compile_units = linkonce constant %llvm.dbg.anchor.type { i32 458752, i32 17 }, section "llvm.metadata"		; 
+
+ at .str = internal constant [4 x i8] c"a.c\00", section "llvm.metadata"		; <[4 x i8]*> [#uses=1]
+ at .str1 = internal constant [6 x i8] c"/tmp/\00", section "llvm.metadata"	; <[6 x i8]*> [#uses=1]
+ at .str2 = internal constant [55 x i8] c"4.2.1 (Based on Apple Inc. build 5636) (LLVM build 00)\00", section "llvm.metadata"		; <[55 x i8]*> [#uses=1]
+ at llvm.dbg.compile_unit = internal constant %llvm.dbg.compile_unit.type { i32 458769, { }* bitcast (%llvm.dbg.anchor.type* @llvm.dbg.compile_units to { }*), i32 1, i8* getelementptr ([4 x i8]* @.str, i32 0, i32 0), i8* getelementptr ([6 x i8]* @.str1, i32 0, i32 0), i8* getelementptr ([55 x i8]* @.str2, i32 0, i32 0), i1 true, i1 false, i8* null }, section "llvm.metadata"		; <%llvm.dbg.compile_unit.type*> [#uses=1]
+
+declare void @llvm.dbg.stoppoint(i32, i32, { }*) nounwind
+
+
+define i32 @test(i1 %C) {
+        br i1 %C, label %T1, label %F1
+
+T1:             ; preds = %0
+call void @llvm.dbg.stoppoint(i32 5, i32 0, { }* bitcast (%llvm.dbg.compile_unit.type* @llvm.dbg.compile_unit to { }*))
+        br label %Cont
+
+F1:             ; preds = %0
+call void @llvm.dbg.stoppoint(i32 5, i32 0, { }* bitcast (%llvm.dbg.compile_unit.type* @llvm.dbg.compile_unit to { }*))
+        br label %Cont
+
+Cont:           ; preds = %F1, %T1
+        %C2 = phi i1 [ false, %F1 ], [ true, %T1 ]              ; <i1> [#uses=1]
+call void @llvm.dbg.stoppoint(i32 5, i32 0, { }* bitcast (%llvm.dbg.compile_unit.type* @llvm.dbg.compile_unit to { }*))
+        br i1 %C2, label %T2, label %F2
+
+T2:             ; preds = %Cont
+call void @llvm.dbg.stoppoint(i32 5, i32 0, { }* bitcast (%llvm.dbg.compile_unit.type* @llvm.dbg.compile_unit to { }*))
+        call void @bar( )
+call void @llvm.dbg.stoppoint(i32 5, i32 0, { }* bitcast (%llvm.dbg.compile_unit.type* @llvm.dbg.compile_unit to { }*))
+        ret i32 17
+
+F2:             ; preds = %Cont
+call void @llvm.dbg.stoppoint(i32 5, i32 0, { }* bitcast (%llvm.dbg.compile_unit.type* @llvm.dbg.compile_unit to { }*))
+        ret i32 1
+}
+
+declare void @bar()
+





More information about the llvm-commits mailing list
--- llvm/trunk/lib/Transforms/Scalar/CondPropagate.cpp (original)
+++ llvm/trunk/lib/Transforms/Scalar/CondPropagate.cpp Thu Feb  5 17:32:52 2009
@@ -17,6 +17,7 @@
 #include "llvm/Constants.h"
 #include "llvm/Function.h"
 #include "llvm/Instructions.h"
+#include "llvm/IntrinsicInst.h"
 #include "llvm/Pass.h"
 #include "llvm/Type.h"
 #include "llvm/Transforms/Utils/BasicBlockUtils.h"
@@ -134,8 +135,8 @@
 // jump directly to the destination instead of going through this block.
 void CondProp::SimplifyPredecessors(BranchInst *BI) {
   // TODO: We currently only handle the most trival case, where the PHI node has
   // one use (the branch), and is the only instruction besides the branch in the
   // block.
   PHINode *PN = cast<PHINode>(BI->getCondition());
 
   if (PN->getNumIncomingValues() == 1) {
@@ -148,7 +149,12 @@
   if (!PN->hasOneUse()) return;
 
   BasicBlock *BB = BI->getParent();
-  if (&*BB->begin() != PN || &*next(BB->begin()) != BI)
+  if (&*BB->begin() != PN)
+    return;
+  BasicBlock::iterator BBI = BB->begin();
+  BasicBlock::iterator BBE = BB->end();
+  while (BBI != BBE && isa<DbgInfoIntrinsic>(++BBI));
+  if (&*BBI != BI)
     return;
 
   // Ok, we have this really simple case, walk the PHI operands, looking for
@@ -176,13 +182,18 @@
 // the destination instead of going through this block.
 void CondProp::SimplifyPredecessors(SwitchInst *SI) {
   // TODO: We currently only handle the most trival case, where the PHI node has
   // one use (the branch), and is the only instruction besides the branch in the
   // block.
   PHINode *PN = cast<PHINode>(SI->getCondition());
   if (!PN->hasOneUse()) return;
 
   BasicBlock *BB = SI->getParent();
-  if (&*BB->begin() != PN || &*next(BB->begin()) != SI)
+  if (&*BB->begin() != PN)
+    return;
+  BasicBlock::iterator BBI = BB->begin();
+  BasicBlock::iterator BBE = BB->end();
+  while (BBI != BBE && isa<DbgInfoIntrinsic>(++BBI));
+  if (&*BBI != SI)
     return;
 
   bool RemovedPreds = false;
@@ -134,8 +135,8 @@
 void CondProp::SimplifyPredecessors(BranchInst *BI) {
   // TODO: We currently only handle the most trival case, where the PHI node has
-  // one use (the branch), and is the only instruction besides the branch in the
-  // block.
+  // one use (the branch), and is the only instruction besides the branch and dbg
+  // intrinsics in the block.
   PHINode *PN = cast<PHINode>(BI->getCondition());
 
@@ -176,13 +182,18 @@
 void CondProp::SimplifyPredecessors(SwitchInst *SI) {
   // TODO: We currently only handle the most trival case, where the PHI node has
-  // one use (the branch), and is the only instruction besides the branch in the
-  // block.
+  // one use (the branch), and is the only instruction besides the branch and 
+  // dbg intrinsics in the block.
   PHINode *PN = cast<PHINode>(SI->getCondition());
   if (!PN->hasOneUse()) return;

Added: llvm/trunk/test/Transforms/CondProp/basictest-dbg.ll
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/test/Transforms/CondProp/basictest-dbg.ll?rev=63898&view=auto

==============================================================================